/**
binds a function to a context

@method bind
@param {Function} func
@param {Object} context
@return {Function}
@private
**/
'use strict';

var bind = function bind(func, context) {
  return function () {
    func.apply(context, Array.prototype.slice.call(arguments));
  };
};

/**
@method each
@param {Array} arr
@param {Function} iterator
@private
**/
var each = function each(arr, iterator, context) {
  context = context || this;

  for (var i = 0, len = arr.length; i < len; i++) {
    iterator.call(context, arr[i], i);
  }
};

/**
@method compact
@param {Array} arr
@private
**/
var compact = function compact(arr) {
  return arr.filter(function (x) {
    return !!x;
  });
};

/**
@method map
@param {Array} arr
@param {Function} iterator
@private
**/

var map = function map(arr, iterator) {
  return arr.map(iterator);
};

/**
@method merge
@return {Object}
@private
**/
var merge = function merge() {
  var result = {},
      arr = Array.prototype.slice.call(arguments, 0);

  each(arr, function (obj) {
    for (var key in obj) {
      if (obj.hasOwnProperty(key)) {
        result[key] = obj[key];
      }
    }
  });

  return result;
};

/**
@method addEvent
@param {Any} host
@param {String} eventName
@param {Function} handler
@param {Any} [context]
@private
**/
var addEvent = function addEvent(host, eventName, handler, context) {
  host.addEventListener(eventName, bind(handler, context), false);
};

/**
@method isArray
@param {Object} o
@return {Boolean}
@private
**/
var isArray = function isArray(o) {
  return Array.isArray(o);
};

/**
@method isFunc
@param {any} val
@return {Boolean}
@private
**/
var isFunc = function isFunc(val) {
  return typeof val === 'function';
};

/**
@method isPlainObject
@param {any} val
@return {Boolean}
@private
**/
var isPlainObject = function isPlainObject(val) {
  return !!val && val.constructor === Object;
};

/**
@method isString
@param {Any} val
@return {Boolean}
@private
**/
var isString = function isString(val) {
  return typeof val === 'string';
};

module.exports = {
  bind: bind,
  each: each,
  compact: compact,
  map: map,
  merge: merge,
  addEvent: addEvent,
  isArray: isArray,
  isFunc: isFunc,
  isPlainObject: isPlainObject,
  isString: isString
};